/*---------------------------------------------------------------------------*
  Project:  NintendoWare
  File:     eft_Particle.fsh
  
  Copyright (C)2011-2012 Nintendo All rights reserved.
  
  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.
  
  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/




//------------------------------------------------------------------------------
// J[vZ
//------------------------------------------------------------------------------
// J[1
#if ( FRAGMENT_COLOR_MODE == FRAGMENT_COLOR_MODE_COLOR )
vec3 GetFragmentColor( vec4 texture, vec4 color0, vec4 color1 )
{
    vec3 outColor;

    outColor.r = color0.r;
    outColor.g = color0.g;
    outColor.b = color0.b;
    
    return outColor;
}
#endif

// J[2
#if ( FRAGMENT_COLOR_MODE == FRAGMENT_COLOR_MODE_TEXTURE )
vec3 GetFragmentColor( vec4 texture, vec4 color0, vec4 color1 )
{
    vec3 outColor;

    outColor.r = texture.r * color0.r;
    outColor.g = texture.g * color0.g;
    outColor.b = texture.b * color0.b;
    
    return outColor;
}
#endif

// J[3
#if ( FRAGMENT_COLOR_MODE == FRAGMENT_COLOR_MODE_TEXTURE_INTERPOLATE )
vec3 GetFragmentColor( vec4 texture, vec4 color0, vec4 color1 )
{
    vec3 outColor;

    outColor.r = ( texture.r * color0.r ) + ( ( 1 - texture.r ) * color1.r );
    outColor.g = ( texture.g * color0.g ) + ( ( 1 - texture.g ) * color1.g );
    outColor.b = ( texture.b * color0.b ) + ( ( 1 - texture.b ) * color1.b );
    
    return outColor;
}
#endif

// J[4
#if ( FRAGMENT_COLOR_MODE == FRAGMENT_COLOR_MODE_NORMAL_ADD )
vec3 GetFragmentColor( vec4 texture, vec4 color0, vec4 color1 )
{
    vec3 outColor;

    outColor.r = ( texture.r * color0.r ) + color1.r;
    outColor.g = ( texture.g * color0.g ) + color1.g;
    outColor.b = ( texture.b * color0.b ) + color1.b;
    
    return outColor;
}
#endif




//------------------------------------------------------------------------------
// At@vZ
//------------------------------------------------------------------------------
// At@1
float GetFragmentAlpha( vec4 texture, float alpha )
{
#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_MOD )
    return  texture.a * alpha;
#endif

#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_SUB )
    return  ( texture.a - ( 1 - alpha ) ) * 2;
#endif

#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_MOD_R )
    return  texture.r * alpha;
#endif

#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_SUB_R )
    return  ( texture.r - ( 1 - alpha ) ) * 2;
#endif
}




//---------------------------------------------------
// 1ڃeNX`J[擾( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_GET_TEXTURE_COLOR
vec4 GetTextureColor( vec2 textureCoord )
{
#if ( ( FRAGMENT_FIRST_TEXTURE_COLOR_OPT == FRAGMENT_PARAMETER_INPUT_OPT_ONE ) && ( FRAGMENT_FIRST_TEXTURE_ALPHA_OPT == FRAGMENT_PARAMETER_INPUT_OPT_ONE ) )
    return vec4( 1.0, 1.0, 1.0, 1.0 );
#else

    vec4 texture = texture2D( s_firstTexture, textureCoord );

#if ( FRAGMENT_FIRST_TEXTURE_COLOR_OPT == FRAGMENT_PARAMETER_INPUT_OPT_ONE )
    texture.x = 1.0;
    texture.y = 1.0;
    texture.z = 1.0;
#endif

#if ( FRAGMENT_FIRST_TEXTURE_ALPHA_OPT == FRAGMENT_PARAMETER_INPUT_OPT_ONE )
    texture.w = 1.0;
#endif

    return texture;

#endif
}
#endif


//---------------------------------------------------
// 2ڃeNX`J[擾( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_GET_MULTI_TEXTURE_COLOR
vec4 GetMultiTextureColor( vec2 textureCoord )
{
#if ( ( FRAGMENT_SECOND_TEXTURE_COLOR_OPT == FRAGMENT_PARAMETER_INPUT_OPT_ONE ) && ( FRAGMENT_SECOND_TEXTURE_ALPHA_OPT == FRAGMENT_PARAMETER_INPUT_OPT_ONE ) )
    return vec4( 1.0, 1.0, 1.0, 1.0 );
#else

    vec4 texture = texture2D( s_secondTexture, textureCoord );

#if ( FRAGMENT_SECOND_TEXTURE_COLOR_OPT == FRAGMENT_PARAMETER_INPUT_OPT_ONE )
    texture.x = 1.0;
    texture.y = 1.0;
    texture.z = 1.0;
#endif

#if ( FRAGMENT_SECOND_TEXTURE_ALPHA_OPT == FRAGMENT_PARAMETER_INPUT_OPT_ONE )
    texture.w = 1.0;
#endif

    return texture;

#endif
}
#endif


//---------------------------------------------------
// v~eBu_J[擾( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_GET_PRIMITIVE_COLOR
vec3 GetPrimitiveColor()
{
#if ( FRAGMENT_PRIMITIVE_COLOR_OPT == FRAGMENT_PARAMETER_INPUT_OPT_SRC )
    return v_outPrimitiveColor.xyz;
#endif
#if ( FRAGMENT_PRIMITIVE_COLOR_OPT == FRAGMENT_PARAMETER_INPUT_OPT_ONE )
    return vec3( 1.0, 1.0, 1.0 );
#endif
}
#endif


//---------------------------------------------------
// v~eBu_At@擾( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_GET_PRIMITIVE_ALPHA
float GetPrimitiveAlpha()
{
#if ( FRAGMENT_PRIMITIVE_ALPHA_OPT == FRAGMENT_PARAMETER_INPUT_OPT_SRC )
    return v_outPrimitiveColor.w;
#endif
#if ( FRAGMENT_PRIMITIVE_ALPHA_OPT == FRAGMENT_PARAMETER_INPUT_OPT_ONE )
    return 1.0;
#endif
}
#endif


//---------------------------------------------------
// fvXl擾( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_GET_DEPTH_VALUE
float GetDepthValue( vec4 position )
{
    float depth = position.z / position.w;
    depth = - ( nearZ * farZ ) / ( depth * ( farZ - nearZ ) - farZ );
    return depth;
}
#endif


//---------------------------------------------------
// fvXeNX`fvXl擾( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_GET_DEPTH_VALUE_FROM_TEXTURE
float GetDepthValueFromTexture( vec4 projection )
{
    float depth = texture2DProj( s_depthBufferTexture, projection ).r;
    depth = - ( nearZ * farZ ) / ( depth  * ( farZ - nearZ ) - farZ );
    return depth;
}
#endif


//---------------------------------------------------
// J[obt@eNX`J[l擾( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_GET_COLOR_VALUE_FROM_TEXTURE
vec4 GetColorValueFromTexture( vec4 projection )
{
    return texture2DProj( s_frameBufferTexture, projection );
}
#endif


//---------------------------------------------------
// eNX`J[uh( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_BLEND_TEXTURE_COLOR
vec4 BlendTextureColor( vec4 texture0, vec4 texture1 )
{
    vec4 retColor;

    // TEXTURE COLOR COMPOSITE
#if ( FRAGMENT_TEXTURE_COLOR_COMPOSITE == FRAGMENT_COMPOSITE_MUL )
    retColor.rgb = texture0.rgb * texture1.rgb;
#endif
#if ( FRAGMENT_TEXTURE_COLOR_COMPOSITE == FRAGMENT_COMPOSITE_ADD )
    retColor.rgb = texture0.rgb + texture1.rgb;
#endif
#if ( FRAGMENT_TEXTURE_COLOR_COMPOSITE == FRAGMENT_COMPOSITE_SUB )
    retColor.rgb = texture0.rgb - texture1.rgb;
#endif

    // TEXTURE ALPHA COMPOSITE
#if ( FRAGMENT_TEXTURE_ALPHA_COMPOSITE == FRAGMENT_COMPOSITE_MUL )
    retColor.a   = texture0.a * texture1.a;
#endif
#if ( FRAGMENT_TEXTURE_ALPHA_COMPOSITE == FRAGMENT_COMPOSITE_ADD )
    retColor.a   = texture0.a + texture1.a;
#endif
#if ( FRAGMENT_TEXTURE_ALPHA_COMPOSITE == FRAGMENT_COMPOSITE_SUB )
    retColor.a   = texture0.a - texture1.a;
#endif

    return retColor;
}
#endif


//---------------------------------------------------
// v~eBuJ[uh( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_BLEND_PRIMITIVE_COLOR
vec3 BlendPrimitiveColor( vec3 outputColor, vec3 primitiveColor )
{
    vec3 retColor;

#if ( FRAGMENT_PRIMITIVE_COLOR_COMPOSITE == FRAGMENT_COMPOSITE_MUL )
    retColor   = outputColor * primitiveColor;
#endif
#if ( FRAGMENT_PRIMITIVE_COLOR_COMPOSITE == FRAGMENT_COMPOSITE_ADD )
    retColor   = outputColor + primitiveColor;
#endif
#if ( FRAGMENT_PRIMITIVE_COLOR_COMPOSITE == FRAGMENT_COMPOSITE_SUB )
    retColor   = outputColor - primitiveColor;
#endif

    return retColor;
}
#endif


//---------------------------------------------------
// v~eBuAt@uh( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_BLEND_PRIMITIVE_ALPHA
vec4 BlendPrimitiveAlpha( vec4 texture, float primitiveAlpha )
{
#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_MOD )
    float retAlpha = texture.a;
#endif
#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_SUB )
    float retAlpha = texture.a;
#endif
#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_MOD_R )
    float retAlpha = texture.r;
#endif
#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_SUB_R )
    float retAlpha = texture.r;
#endif


#if ( FRAGMENT_PRIMITIVE_ALPHA_COMPOSITE == FRAGMENT_COMPOSITE_MUL )
    retAlpha   = retAlpha * primitiveAlpha;
#endif
#if ( FRAGMENT_PRIMITIVE_ALPHA_COMPOSITE == FRAGMENT_COMPOSITE_ADD )
    retAlpha   = retAlpha + primitiveAlpha;
#endif
#if ( FRAGMENT_PRIMITIVE_ALPHA_COMPOSITE == FRAGMENT_COMPOSITE_SUB )
    retAlpha   = retAlpha - primitiveAlpha;
#endif


#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_MOD )
    texture.a = retAlpha;
#endif
#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_SUB )
    texture.a = retAlpha;
#endif
#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_MOD_R )
    texture.r = retAlpha;
#endif
#if ( FRAGMENT_ALPHA_MODE == FRAGMENT_ALPHA_MODE_SUB_R )
    texture.r = retAlpha;
#endif

    return texture;
}
#endif



//---------------------------------------------------
// [U[Ǝ̃tOgo̓J[ŏI( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_FINAL_ADJUSTMENT_FRAGMENT_COLOR
void FinalAdjustmentFragmentColor(){}
#endif


//---------------------------------------------------
// \tgGbW( JX^}CY\ )
//---------------------------------------------------
#ifndef USE_USR_SOFT_EDGE
void CalculateSoftEdge()
{
    // sNZZ擾
    float pz = GetDepthValue( v_outPos );

    // eNX`Zl擾
    float tz = GetDepthValueFromTexture( v_outDepthTexCoordProj );

    // eNX`̋PxvZ
    float luminance = dot( OUTPUT_COLOR.rgb, vec3( 0.298912, 0.586611, 0.114478 ) );
    luminance = - ( nearZ * farZ ) / ( clamp( luminance * softEdgeVolume, 0, 1 )  * ( farZ - nearZ ) - farZ );
    luminance *= step( 0.001, softEdgeVolume );

    // V[Z/sNZZ/{[llăAt@l肷
    float dis = tz - ( pz - luminance );

    // At@
    dis /= softEdgeDist;
    dis = clamp( dis, 0.0, 1.0 );
    OUTPUT_COLOR.a *= mix( 0.0, 1.0, dis );
}
#endif




//------------------------------------------------------------------------------
// ʏp[eBN  \tgp[eBN
//------------------------------------------------------------------------------
#if ( FRAGMENT_PARTICLE_SHADER_MODE == FRAGMENT_PARTICLE_SHADER_MODE_NORMAL )
void main()
{
    //------------------------------------------------------------------------------
    // ʏp[eBNƂĂ̏
    //------------------------------------------------------------------------------
    // eNX`J[擾
    vec4 texture = GetTextureColor( v_outTexCoord );


    //------------------------------------------------------------------------------
    // eNX` R|Wbg
    //------------------------------------------------------------------------------
#if ( FRAGMENT_TEXTURE_MODE == FRAGMENT_TEXTURE_MODE_SECOND )

    // QڃeNX`J[擾
    vec4 scondTexture = GetMultiTextureColor( v_outMultiTexCoord );

    texture = BlendTextureColor( texture, scondTexture );

#endif


    // J[
    OUTPUT_COLOR.rgb = GetFragmentColor( texture, v_outColor0, v_outColor1 );

    // v~eBuJ[f
#ifdef MESH_TYPE_PRIMITIVE
    OUTPUT_COLOR.rgb = BlendPrimitiveColor( OUTPUT_COLOR.rgb, GetPrimitiveColor() );
#endif


    // At@
#ifdef MESH_TYPE_PRIMITIVE
    texture = BlendPrimitiveAlpha( texture, GetPrimitiveAlpha() );
#endif
    OUTPUT_COLOR.a = GetFragmentAlpha( texture, v_outColor0.a );


    //------------------------------------------------------------------------------
    // \tgGbW
    //------------------------------------------------------------------------------
#if ( FRAGMENT_PARTICLE_SHADER_SOFT_EDGE == FRAGMENT_SOFT_EDGE_USE )
    CalculateSoftEdge();
#endif


   // tOg̃JX^VF[_
   FinalAdjustmentFragmentColor();
}
#endif




//------------------------------------------------------------------------------
// ܃p[eBN
//------------------------------------------------------------------------------
#if ( FRAGMENT_PARTICLE_SHADER_MODE == FRAGMENT_PARTICLE_SHADER_MODE_REFRECT )
void main()
{
    // eNX`J[(cݒl)擾
    vec4 texture = GetTextureColor( v_outTexCoord ) * 2.0 - 1.0;
    texture *= v_outFrameTexCoordProj.w;


    texture.x *= shaderParam0;
    texture.y = texture.w * shaderParam1;
    texture.z = 0.0;
    texture.w = 0.0;


    // [vlvZ
#if ( FRAGMENT_TEXTURE_MODE == FRAGMENT_TEXTURE_MODE_FIRST )
    vec4 waap = v_outFrameTexCoordProj + texture;
#endif
#if ( FRAGMENT_TEXTURE_MODE == FRAGMENT_TEXTURE_MODE_SECOND )
    float waapa = texture2D( s_secondTexture, v_outMultiTexCoord ).a;

#ifdef REFRECTION_APPLY_ALPHA
    vec4 waap = v_outFrameTexCoordProj + texture * waapa * v_outColor0.a;
#else
    vec4 waap = v_outFrameTexCoordProj + texture;
#endif
#endif


    // t[obt@eNX`J[擾
    vec4 frame = GetColorValueFromTexture( waap );
    frame.a = 1.0f;


    // J[
    OUTPUT_COLOR.rgb = GetFragmentColor( frame, v_outColor0, v_outColor1 );

    // v~eBuJ[f
#ifdef MESH_TYPE_PRIMITIVE
    OUTPUT_COLOR.rgb = BlendPrimitiveColor( OUTPUT_COLOR.rgb, GetPrimitiveColor() );
#endif


    // At@
#ifdef MESH_TYPE_PRIMITIVE
    frame = BlendPrimitiveAlpha( frame, GetPrimitiveAlpha() );
#endif
    OUTPUT_COLOR.a = GetFragmentAlpha( frame, v_outColor0.a );

#if ( FRAGMENT_TEXTURE_MODE == FRAGMENT_TEXTURE_MODE_SECOND )
    OUTPUT_COLOR.a = OUTPUT_COLOR.a * waapa;
#endif

#if ( FRAGMENT_TEXTURE_MODE == FRAGMENT_TEXTURE_MODE_SECOND )
    OUTPUT_COLOR.a *= frame.a;
#endif

    //------------------------------------------------------------------------------
    // \tgGbW
    //------------------------------------------------------------------------------
#if ( FRAGMENT_PARTICLE_SHADER_SOFT_EDGE == FRAGMENT_SOFT_EDGE_USE )
    CalculateSoftEdge();
#endif

   // tOg̃JX^VF[_
   FinalAdjustmentFragmentColor();
}
#endif





//------------------------------------------------------------------------------
// c݃p[eBN
//------------------------------------------------------------------------------
#if ( FRAGMENT_PARTICLE_SHADER_MODE == FRAGMENT_PARTICLE_SHADER_MODE_DISTORTION )
void main()
{
    // eNX`J[(cݒl)擾
    vec4 texture = GetTextureColor( v_outTexCoord ) * 2.0 - 1.0;


    // [vl
    vec2 waap;
    waap.x = v_outMultiTexCoord.x + texture.x * shaderParam0;
    waap.y = v_outMultiTexCoord.y + texture.w * shaderParam1;


    // TueNX`J[擾
    vec4 subTexture = GetMultiTextureColor( waap );


    // J[
    OUTPUT_COLOR.rgb = GetFragmentColor( subTexture, v_outColor0, v_outColor1 );

    // v~eBuJ[f
#ifdef MESH_TYPE_PRIMITIVE
    OUTPUT_COLOR.rgb = BlendPrimitiveColor( OUTPUT_COLOR.rgb, GetPrimitiveColor() );
#endif


    // At@
#ifdef MESH_TYPE_PRIMITIVE
    subTexture = BlendPrimitiveAlpha( subTexture, GetPrimitiveAlpha() );
#endif
    OUTPUT_COLOR.a = GetFragmentAlpha( subTexture, v_outColor0.a );


    //------------------------------------------------------------------------------
    // \tgGbW
    //------------------------------------------------------------------------------
#if ( FRAGMENT_PARTICLE_SHADER_SOFT_EDGE == FRAGMENT_SOFT_EDGE_USE )
    CalculateSoftEdge();
#endif

   // tOg̃JX^VF[_
   FinalAdjustmentFragmentColor();
}
#endif


